#!/usr/bin/env bash

#------------------
# Global variables
#-----------------
user="unknown"
installDir="/usr/local/harris/"
scriptDir=$(pwd)
tempDir=$(mktemp -d)

_isSpecial="0"
_platform=""
_platformExt=""
_platformBinDir=""
_installPath=""
_currentVersion=""

# initializes all the platform variables
initializePlatform() {
  unameOut="$(uname -s)"
  if [[ "${unameOut}" == *"Darwin"* ]]; then
    _platform="darwin"
    _platformExt=".dylib"
  else
    _platform="linux"
    _platformExt=".so"
  fi
  _platformBinDir="bin/bin.${_platform}.x86_64"
}

unpackData() {
  cd "${tempDir}"

  if [[ "$_platform" == "linux" ]]; then
    dataPack="${scriptDir}/${_platform}.data.tgz"
  else
    dataPack="${scriptDir}/${_platform}.data.zip"
  fi

  if [[ ! -f ${dataPack} ]]; then
    echo "The data required to install this patch is missing. ${dataPack}"
    echo "Exiting the installation."
    exit 1
  fi

  if [[ "$_platform" == "linux" ]]; then
    tar -xf ${dataPack} --no-same-owner
  else
    unzip ${dataPack}
  fi

  if [ "$?" != "0" ]; then
    echo "Error extracting files from ${dataPack} to ${tempDir}"
    echo "Exiting the installation."
    exit 1
  fi

  # rename this file to simplify refactoring
  mv "sav/license_administrator_no_license.sav" "sav/license_administrator.sav"

  cd "${scriptDir}"
}

#----
# Show the license
# Ask the installer to agree with the license.
#--------
agreeToLicense() {
  if [ ${SILENT_INSTALL} == 1 ]; then
    return 0
  fi

  echo ""
  echo "***********************************************************"
  echo "*  License Agreement"
  echo "***********************************************************"
  echo ""

  more ${curdir}/legal/eula.txt

  # Ask the installer if they accept the license agreement
  answer=""
  while [[ ! ("$answer" == "y" || "$answer" == "n") ]]
  do
    echo -n "Do you accept all of the terms of the preceding license agreement? (y/n):  "
    read answer
    answer=`echo ${answer} | tr '[:upper:]' '[:lower:]'`
  done

  if [ "$answer" != "y" ]; then
    echo "Exiting installation."
    echo ""
    exit 1 # fail
  fi
}

# THIS DOES NOT CURRENTLY TAKE DELIMITERS
# Mmp is the format 560, 561, etc
# -1 if left is less than, 0 equal, 1 if left greater than
_versionCompareResult=0
versionCompare() {
  left=$1
  right=$2

  for i in {0..2}; do
    l=${left:${i}:1}
    r=${right:${i}:1}

    if [[ l -lt r ]]; then
      _versionCompareResult=-1
      return 0
    fi

    if [[ l -gt r ]]; then
      _versionCompareResult=1
      return 0
    fi
  done

  # equal
  _versionCompareResult=0
  return 0
}

# INCLUSIVE min and max
# checks that the given version is in the range
# inputs: version min max
# example: versionInRange "871" "860" "880"
# returns 0 on true, -1 if less than, 1 if greater than
_versionInRangeResult=0
versionInRange() {
  ver=$1
  minVer=$2
  maxVer=$3

  versionCompare ${ver} ${minVer}
  if [ "${_versionCompareResult}" == "-1" ]; then
    _versionInRangeResult=-1
    return 0
  fi

  versionCompare ${ver} ${maxVer}
  if [ "${_versionCompareResult}" == "1" ]; then
    _versionInRangeResult=1
    return 0
  fi

  _versionInRangeResult=0
  return 0
}

# returns 1 if invalid, 0 if valid
# global _isSpecial is overwritten on call
# _installPath set to the install path
# _currentVersion is set to the 3 character current version: example: 880
getValidIDLInstallPath() {
  _isSpecial="0"
  inputPath="$1"

  product=""
  if [[ "${inputPath}" == *"idl8"* ]]; then
    product="idl"
  elif [[ "${inputPath}" == *"envi5"* ]]; then
    product="envi"
  else
    _installPath=""
    return "1" # not product folder
  fi

  if [ ! -f "${inputPath}/version.txt" ]; then
    _installPath=""
    return "1" # not product folder
  fi

  ver=$(cat "${inputPath}/version.txt")
  if [ "${#ver}" == "2" ]; then
    ver="${ver}0"
  fi
  _currentVersion="${ver}"

  if [ "${product}" == "idl" ]; then
    versionCompare ${ver} "860"
    if [ "${_versionCompareResult}" == "0" ]; then
      _isSpecial="1"
    fi

    versionInRange ${ver} "860" "883"
    if [ "${_versionInRangeResult}" == "0" ]; then
      _installPath=${inputPath}
      return 0
    fi
  else
    versionCompare ${ver} "540"
    if [ "${_versionCompareResult}" == "0" ]; then
      _isSpecial="1"
    fi

    versionInRange ${ver} "540" "563"
    if [ "${_versionInRangeResult}" == "0" ]; then
      if [ "${ver:0:2}" == "54" ]; then
        _installPath="${inputPath}/idl86"
      elif [ "${ver:0:2}" == "55" ]; then
        _installPath="${inputPath}/idl87"
      elif [ "${ver:0:2}" == "56" ]; then
        _installPath="${inputPath}/idl88"
      fi
      return 0
    fi
  fi

  displayVersion="${ver:0:1}.${ver:1:1}"
  subminor=${ver:2:1}
  if [ "${subminor}" != "0" ]; then
    displayVersion="${displayVersion}.${subminor}"
  fi
  echo "Warning: Version is not in expected range and will not be patched: ${displayVersion} [${inputPath}]"
  _installPath=""
  return 1
}

# create a backup of an existing file
# does nothing if the backup already exists
# source file
createBackup() {
  srcFile=$1
  backupFile="${srcFile}.bak"
  if [ ! -f ${backupFile} ]; then
    cp -pf "${srcFile}" "${backupFile}" >/dev/null 2>/dev/null
  fi
  if [ -f ${backupFile} ]; then
    rm -f "${srcFile}" >/dev/null 2>/dev/null 
  fi
  return 0
}

# install a single file from the temp directory
# filename, source dir, dest dir
installFile() {
  fileName=$1
  src="${tempDir}/$2"
  dst=$3
  cp -pf "${src}/${fileName}" "${dst}/${fileName}"
  return $?
}

# installs the entire patch
# idl directory
installPatch() {
  rootDir=$1
  binDir="${rootDir}/${_platformBinDir}"

  # older versions require more updates
  versionInRange ${_currentVersion} "860" "880" # IDL
  if [ "$_versionInRangeResult" != "0" ]; then
    versionInRange ${_currentVersion} "540" "560" # ENVI
  fi
  requiresCllPatch=$_versionInRangeResult

  createBackup "${rootDir}/lib/hook/license_administrator.sav"
  if [ "$requiresCllPatch" == "0" ]; then
    createBackup "${binDir}/libcll${_platformExt}"
    createBackup "${rootDir}/lib/datatypes/dictionary.sav"
    createBackup "${rootDir}/lib/datatypes/orderedhash.sav"

    # backup these two files so they're not accidently accessed
    createBackup "${licenseUtils}/activate"
    createBackup "${licenseUtils}/libcll${_platformExt}"

    # patch json_parse if pre-idl 871
    versionInRange ${_currentVersion} "860" "870" # IDL
    if [ "$_versionInRangeResult" != "0" ]; then
      versionInRange ${_currentVersion} "540" "550" # ENVI
    fi

    if [ "$_versionInRangeResult" == "0" ]; then
      installFile "json_parse_legacy.sav" "sav" "${rootDir}/lib/hook"
    fi

    installFile "dictionary.sav" "sav" "${rootDir}/lib/datatypes"
    installFile "orderedhash.sav" "sav" "${rootDir}/lib/datatypes"

    if [ "${_isSpecial}" == "0" ]; then
      installFile "libcll${_platformExt}" "${_platform}.bin" "${binDir}"
    else
      installFile "libcll${_platformExt}" "${_platform}.bin86" "${binDir}"
    fi

    # remove -novm from the harrislicense executeable
    harrisLicenseExe="${rootDir}/bin/harrislicense"
    $(sed -i.bak "s/-novm//g" ${harrisLicenseExe})
  else
    # newer installs already have an alan installed
    createBackup "${binDir}/libalan${_platformExt}"
  fi

  # install the new license admin and alan library
  installFile "license_administrator.sav" "sav" "${rootDir}/lib/hook"
  installFile "libalan${_platformExt}" "${_platform}.bin" "${binDir}"

  # install the alan 'licensing' utility and friends
  licenseUtils="${rootDir}/license_utils/bin.${_platform}.x86_64"
  installFile "libalan${_platformExt}" "${_platform}.bin" "${licenseUtils}"
  installFile "licensing" "${_platform}.bin" "${licenseUtils}"
  installFile "licensing.cfg" "${_platform}.bin" "${licenseUtils}"

  # ... and install required GCC runtime libraries to support older Linux versions
  if [[ "$_platform" == "linux" ]]; then
    installFile "libstdc++.so" "${_platform}.bin" "${binDir}"
    installFile "libstdc++.so.6" "${_platform}.bin" "${binDir}"
    installFile "libstdc++.so.6.0.30" "${_platform}.bin" "${binDir}"
    installFile "libstdc++.so" "${_platform}.bin" "${licenseUtils}"
    installFile "libstdc++.so.6" "${_platform}.bin" "${licenseUtils}"
    installFile "libstdc++.so.6.0.30" "${_platform}.bin" "${licenseUtils}"
    installFile "gcc-runtime-12.1.0.zip" "help/legal/source" "${rootDir}/help/legal/source"
  fi

  return 0
}

# trys to install to a given path
# validates the path is a valid install location first
# directory
tryInstall() {
  inputPath=$1
  inputPath=$( cd "${inputPath}" && pwd )
  skipPrompt=$2

  getValidIDLInstallPath "${inputPath}"
  ok=$?
  if [ "${ok}" == "1" ]; then
    return 1
  fi

  currentInstallPath=$( cd "${_installPath}" && pwd )
  if [ "${skipPrompt}" == "0" ]; then
    major=${_currentVersion:0:1}
    minor=${_currentVersion:1:1}
    subminor=${_currentVersion:2:1}
    productName=""
    if [ "$major" == "5" ]; then
      idlMinor=$(($minor + 2))
      if [ "${subminor}" == "0" ]; then
        productName="ENVI 5.${minor} / IDL 8.${idlMinor}"
      else
        productName="ENVI 5.${minor}.${subminor} / IDL 8.${idlMinor}.${subminor}"
      fi
    elif [ "$major" == "8" ]; then
      if [ "${subminor}" == "0" ]; then
        productName="IDL 8.${minor}"
      else
        productName="IDL 8.${minor}.${subminor}"
      fi
    fi
    answer=""
    while [[ ! ("$answer" == "y" || "$answer" == "n") ]]
    do
      echo -n "Apply licensing patch for ${productName} [${inputPath}]? (y/n): "
      read answer
      answer=`echo ${answer} | tr '[:upper:]' '[:lower:]'`
    done

    if [ "$answer" != "y" ]; then
      return 1
    fi
  fi

  installPatch $currentInstallPath
  ok=$?
  if [ "${ok}" == "0" ]; then
    echo "$currentInstallPath patched successfully."
    return 0
  fi

  return 1
}

handleInstallDir() {
  echo ""
  echo "***********************************************************"
  echo "*  Installation Directory"
  echo "***********************************************************"
  echo ""
  echo "Please enter a directory containing one or more ENVI or IDL installations [${installDir}]:"
  read dirName

  if [ "$dirName" != "" ]; then
    installDir=$dirName
  fi

  installDir=$( cd "${installDir}" && pwd )

  if [ ! -d  "$installDir" ]; then
      echo "
The ${installDir} directory does not exist."
      echo "Exiting the installation."
      return 1
  fi

  if [ -f "$installDir/version.txt" ]; then
    # this is a specific envi / idl install
    tryInstall $installDir 1
    if [ "$?" != "0" ]; then
      echo "
Failed to patch installation ${currentInstallPath}"
      echo "Exiting the installation."
      return 1
    fi
  else
    installSuccess="1"
    if [ -d "$installDir/idl86" ]; then
      tryInstall "$installDir/idl86" 0
      if [ "$?" == "0" ]; then
        installSuccess="0"
      fi
    fi
    
    if [ -d "$installDir/idl87" ]; then
      tryInstall "$installDir/idl87" 0
      if [ "$?" == "0" ]; then
        installSuccess="0"
      fi
    fi

    if [ -d "$installDir/idl88" ]; then
      tryInstall "$installDir/idl88" 0
      if [ "$?" == "0" ]; then
        installSuccess="0"
      fi
    fi
    
    if [ -d "$installDir/envi54" ]; then
      tryInstall "$installDir/envi54" 0
      if [ "$?" == "0" ]; then
        installSuccess="0"
      fi
    fi
    
    if [ -d "$installDir/envi55" ]; then
      tryInstall "$installDir/envi55" 0
      if [ "$?" == "0" ]; then
        installSuccess="0"
      fi
    fi
    
    if [ -d "$installDir/envi56" ]; then
      tryInstall "$installDir/envi56" 0
      if [ "$?" == "0" ]; then
        installSuccess="0"
      fi
    fi

    if [ "${installSuccess}" != "0" ]; then
      echo "
No ENVI or IDL installs have been patched."
      echo "Exiting the installer."
      exit 1
    fi
  fi
}

#--------
# Install the patch
#--------
runInstaller(){
  # initialize global platform variables
  initializePlatform

  # show license.txt
  agreeToLicense

  # set umask to retain permissions of extracted tar files
  umask 0000

  # unpack pre-install dependencies
  unpackData

  # ask for installation directory.
  handleInstallDir

  # remove preinstall data
  rm -rf "${tempDir}"

  echo ""
  echo "***********************************************************"
  echo "Installation is Complete."
  echo "***********************************************************"
}

#================================================================
#   BEGIN THE INSTALLATION
#================================================================

SILENT_INSTALL=0

while getopts ":s" opt; do
  case ${opt} in
    s)
      SILENT_INSTALL=1
      ;;
    \?)
      echo "Invalid option: -$OPTARG" >&2
      ;;
  esac
done

echo ""
echo "***********************************************************"
echo "*  Installation of L3Harris Next-Generation License Patch"
echo "***********************************************************"
echo ""
curdir="$(cd "$(dirname "$0")" && pwd)"
pushd ${curdir} > /dev/null

runInstaller

popd > /dev/null
exit 0 # success
